import os
import shutil
import signal
import time

def _init_prefix():
    return '''
<config>
    <parent-provides>
        <service name="CPU"/>
        <service name="File_system"/>
        <service name="LOG"/>
        <service name="Nic"/>
        <service name="PD"/>
        <service name="RM"/>
        <service name="ROM"/>
        <service name="Rtc"/>
        <service name="Timer"/>
    </parent-provides>
    <default-route>
        <any-service> <parent/> <any-child/> </any-service>
    </default-route>
    <default caps="100"/>
'''


def _init_suffix():
    return '</config>'


def _init_broker_node(name, cmd):
    node = '''
    <start name="%s" caps="200">
        <binary name="mosquitto_broker"/>
        <resource name="RAM" quantum="8M"/>
        <config>
''' % name

    for arg in cmd:
            node += '            <arg value="%s"/>\n' % arg

    node += '''
            <libc rtc="/dev/rtc" stdin="/dev/null" stdout="/dev/log" 
                    stderr="/dev/log" socket="/dev/socket" cwd="/test/broker/"/>
            <vfs>
                <dir name="dev">
                    <rtc/>
                    <dir name="socket">
                        <lwip ip_addr="169.254.70.254" netmask="255.255.255.0" gateway="169.254.70.1"/>
                    </dir>
                    <log/> <null/>
                </dir>
                <dir name="etc">
                    <inline name="hosts">127.0.0.1	localhost</inline>
                </dir>
                <dir name="test">
                    <fs label="test"/>
                </dir>
            </vfs>
        </config>
        <route>
            <any-service> <parent/> <any-child/> </any-service>
        </route>
    </start>
'''
    return node


class ConfigError(Exception):
    def __init__(self, message='Mismatched packets'):
        self.message = message


class GenodeBroker:
    brokers = {}
    conf_dir = str(os.environ.get('MOSQ_RUN_DIR')) + '/../configs/'
    init_conf = conf_dir + "init.config"
    number = 0

    def __init__(self, filename, clean_after):
        if os.environ.get('MOSQ_RUN_DIR') is None:
            raise ConfigError("Environment variable MOSQ_RUN_DIR is not provided")

        self.filename = filename
        try:
            shutil.copyfile(filename, GenodeBroker.conf_dir + filename)
        except:
            pass

        self.name = 'broker_%d' % GenodeBroker.number
        self.clean_after = clean_after
        self.cmd = []
        GenodeBroker.number = GenodeBroker.number + 1

    def _write_init(self):
        with open(GenodeBroker.init_conf, 'w') as file:
            file.write(_init_prefix())
            for name, cmd in GenodeBroker.brokers.items():
                file.write(_init_broker_node(name, cmd))

            file.write(_init_suffix())

    def start(self, cmd):
        self.cmd = cmd
        GenodeBroker.brokers[self.name] = cmd
        self._write_init()

    def terminate(self):
        GenodeBroker.brokers.pop(self.name)
        self._write_init()
        if self.clean_after:
            os.remove(self.filename)

    def wait(self, timeout=0.5):
        time.sleep(timeout)

    def communicate(self, timeout=0):
        return (b"", b"")
    
    def send_signal(self, sig):
        if sig == signal.SIGHUP:
            self.terminate()
            self.wait()
            self.start(self.cmd)
